@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2019 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2020 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -296,7 +296,7 @@ namespace KeePass.Forms
 		{
 			m_pwDatabase.SettingsChanged = DateTime.UtcNow;
 
-			if(!m_tbDbName.Text.Equals(m_pwDatabase.Name))
+			if(m_tbDbName.Text != m_pwDatabase.Name)
 			{
 				m_pwDatabase.Name = m_tbDbName.Text;
 				m_pwDatabase.NameChanged = DateTime.UtcNow;
@@ -305,13 +305,13 @@ namespace KeePass.Forms
 			string strNew = m_tbDbDesc.Text;
 			string strOrgFlt = StrUtil.NormalizeNewLines(m_pwDatabase.Description, false);
 			string strNewFlt = StrUtil.NormalizeNewLines(strNew, false);
-			if(!strNewFlt.Equals(strOrgFlt))
+			if(strNewFlt != strOrgFlt)
 			{
 				m_pwDatabase.Description = strNew;
 				m_pwDatabase.DescriptionChanged = DateTime.UtcNow;
 			}
 
-			if(!m_tbDefaultUser.Text.Equals(m_pwDatabase.DefaultUserName))
+			if(m_tbDefaultUser.Text != m_pwDatabase.DefaultUserName)
 			{
 				m_pwDatabase.DefaultUserName = m_tbDefaultUser.Text;
 				m_pwDatabase.DefaultUserNameChanged = DateTime.UtcNow;
