@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2019 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2020 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -447,13 +447,12 @@ namespace KeePassLib.Native
 				Marshal.FreeHGlobal(kvpPointers.Value);
 		}
 
-		internal static Type GetUwpType(string strType)
-		{
-			if(string.IsNullOrEmpty(strType)) { Debug.Assert(false); return null; }
-
-			// https://referencesource.microsoft.com/#mscorlib/system/runtime/interopservices/windowsruntime/winrtclassactivator.cs
-			return Type.GetType(strType + ", Windows, ContentType=WindowsRuntime", false);
-		}
+		// internal static Type GetUwpType(string strType)
+		// {
+		//	if(string.IsNullOrEmpty(strType)) { Debug.Assert(false); return null; }
+		//	// https://referencesource.microsoft.com/#mscorlib/system/runtime/interopservices/windowsruntime/winrtclassactivator.cs
+		//	return Type.GetType(strType + ", Windows, ContentType=WindowsRuntime", false);
+		// }
 
 		// Cf. DecodeArgsToData
 		internal static string EncodeDataToArgs(string strData)
