@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2019 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2020 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -82,6 +82,8 @@ namespace KeePass.Util
 		public static event EventHandler<AutoTypeEventArgs> FilterSendPre;
 		public static event EventHandler<AutoTypeEventArgs> FilterSend;
 
+		public static event EventHandler<AutoTypeEventArgs> SendPost;
+
 		public static event EventHandler<SequenceQueryEventArgs> SequenceQueryPre;
 		public static event EventHandler<SequenceQueryEventArgs> SequenceQuery;
 		public static event EventHandler<SequenceQueryEventArgs> SequenceQueryPost;
@@ -202,6 +204,8 @@ namespace KeePass.Util
 						ex.Message;
 				}
 
+				if(AutoType.SendPost != null) AutoType.SendPost(null, args);
+
 				if(!string.IsNullOrEmpty(strError))
 				{
 					try
